import { LitElement, nothing } from 'lit';
declare const UUIRangeSliderElement_base: (new (...args: any[]) => import("@umbraco-ui/uui-base/lib/mixins").FormControlMixinInterface) & typeof LitElement;
/**
 * @element uui-range-slider
 * @description - Range slider with two handles. Use uui-slider for a single handle.
 * @fires UUIRangeSliderEvent#input on input
 * @fires UUIRangeSliderEvent#change on change
 */
export declare class UUIRangeSliderElement extends UUIRangeSliderElement_base {
    #private;
    static styles: import("lit").CSSResult[];
    static readonly formAssociated = true;
    /**
     * Disables the input.
     * @type {boolean}
     * @attr
     * @default false
     */
    disabled: boolean;
    /**
     * Label to be used for aria-label and eventually as visual label. Adds " low value" and " high value" endings for the two values.
     * @type {string}
     * @attr
     */
    label: String;
    /**
     * This reflects the behavior of a native input step attribute.
     * @type {number}
     * @attr
     * @default 1
     */
    step: number;
    /**
     * Hides the numbers representing the value of each steps. Dots will still be visible
     * @type {boolean}
     * @attr 'hide-step-values'
     * @default false
     */
    hideStepValues: boolean;
    /**
     * Sets the minimum allowed value.
     * @type {number}
     * @attr min
     * @default 0
     */
    min: number;
    /**
     * Sets the maximum allowed value.
     * @type {number}
     * @attr max
     * @default 100
     */
    max: number;
    /**
     * Minimum value gap between the the two picked values. Cannot be lower than the step value and cannot be higher than the maximum gap
     * @type {number}
     * @attr min-gap
     * @default undefined
     */
    minGap?: number;
    /**
     * Maximum value gap between the the two picked values. Cannot be lower than the minimum gap.
     * @type {number}
     * @attr max-gap
     * @default undefined
     */
    maxGap?: number;
    /**
     * This is a value property of the uui-range-slider. Split the two values with comma, forexample 10,50 sets the values to 10 and 50.
     * @type {string}
     * @attr
     * @default 0,100
     */
    get value(): FormDataEntryValue | FormData;
    set value(newVal: FormDataEntryValue | FormData);
    private _valueLow;
    /**
     * The lower picked value.
     * @type {number}
     * @attr value-low
     * @default 0
     */
    set valueLow(newLow: number);
    get valueLow(): number;
    private _valueHigh;
    /**
     * The higher picked value.
     * @type {number}
     * @attr value-high
     * @default 100
     */
    set valueHigh(newHigh: number);
    get valueHigh(): number;
    private _trackWidth;
    private _currentInputFocus?;
    private _currentThumbFocus;
    private _grabbingBoth?;
    private _startPos;
    private _startLow;
    private _startHigh;
    private _inputLow;
    private _inputHigh;
    private _outerTrack;
    private _innerTrack;
    private _thumbLow;
    private _thumbHigh;
    private _innerColor;
    private _bothThumbsTarget;
    protected getFormElement(): HTMLInputElement;
    focus(): void;
    private _onKeypress;
    /** Thumb position */
    private _sliderLowThumbPosition;
    private _sliderHighThumbPosition;
    /** Coloring of the line between thumbs */
    private _fillColor;
    /** Moving thumb */
    private _moveThumb;
    /** Mouse events */
    private _onMouseDown;
    private _onMouseMove;
    private _onMouseUp;
    /** Touch / mobile events */
    private _onTouchStart;
    private _onTouchMove;
    private _onTouchEnd;
    /** */
    private _stop;
    /** The latest thumb in use */
    private _setThumb;
    private _setValueBasedOnCurrentThumb;
    /** Get the value depends on where clicked/touched */
    private _getValue;
    private _validateLowByMinGap;
    private _validateLowByMaxGap;
    private _validateHighByMinGap;
    private _validateHighByMaxGap;
    private _validateValueBasedOnCurrentThumb;
    /** Methods when moving both thumbs */
    private _saveStartPoint;
    private _moveBoth;
    private _getValidatedValues;
    /** CHANGE AND INPUT EVENT LISTENERS */
    private _onChange;
    private _onLowInput;
    private _onHighInput;
    /** Constructor */
    constructor();
    connectedCallback(): void;
    updated(changedProperties: Map<string | number | symbol, unknown>): void;
    /** RENDER */
    render(): import("lit-html").TemplateResult<1>;
    renderNativeInputs(): import("lit-html").TemplateResult<1>;
    renderThumbs(): import("lit-html").TemplateResult<1>;
    /** RENDER STEPS & STEP VALUES */
    renderStepsOutput(): import("lit-html").TemplateResult<1>;
    renderSteps(): import("lit-html").TemplateResult<2>;
    renderStepValues(): import("lit-html").TemplateResult<1> | typeof nothing;
}
declare global {
    interface HTMLElementTagNameMap {
        'uui-range-slider': UUIRangeSliderElement;
    }
}
export {};
